@@ -1,6 +1,6 @@
 /*
 %
-% $Id: biosig-dev.h,v 1.17 2009/04/08 12:49:54 schloegl Exp $
+% $Id$
 % Copyright (C) 2005,2006,2007,2008,2009,2011 Alois Schloegl <a.schloegl@ieee.org>
 % This file is part of the "BioSig for C/C++" repository 
 % (biosig4c++) at http://biosig.sf.net/ 
@@ -34,8 +34,6 @@
 #ifndef __BIOSIG_INTERNAL_H__
 #define __BIOSIG_INTERNAL_H__
 
-#include "biosig.h"
-
 #include <math.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -43,6 +41,51 @@
 #include <time.h>
 
 
+#ifdef NDEBUG
+#define VERBOSE_LEVEL 0		// turn off debugging information
+#else
+extern int VERBOSE_LEVEL; 	// used for debugging
+#endif
+
+
+/*
+	Including ZLIB enables reading gzipped files (they are decompressed on-the-fly)
+	The output files can be zipped, too.
+ */
+
+#ifdef WITH_ZLIB
+#include <zlib.h>
+#ifndef ZLIB_H
+    #if defined(__MINGW64__)
+	#include "win64/zlib/zlib.h"
+    #elif defined(__MINGW32__)
+	#include "win32/zlib/include/zlib.h"
+    #endif
+#endif
+#endif
+
+#ifdef WITH_CHOLMOD
+    #ifdef __APPLE__
+        #include <ufsparse/cholmod.h>
+    #else
+        #include <suitesparse/cholmod.h>
+    #endif
+#endif
+
+#ifdef WITH_HDF5
+    #include <hdf5.h>
+#endif
+#ifdef WITH_NIFTI
+    #include <nifti1.h>
+#endif
+
+
+#ifdef WITH_GSL
+    #include <gsl/gsl_matrix_double.h>
+#endif
+
+#include "biosig.h"
+
 #ifdef	__WIN32__
 #define FILESEP '\\'
 char *getlogin (void);
@@ -293,10 +336,10 @@ void bef64a(  double i, uint8_t* r);
 # define INFINITY (1.0/0.0)   /* positive infinity */
 #endif
 #ifndef isfinite
-# define isfinite(a) (-INFINITY < a && a < INFINITY)
+# define isfinite(a) (-INFINITY < (a) && (a) < INFINITY)
 #endif
 #ifndef isnan
-# define isnan(a) (a!=a)
+# define isnan(a) ((a)!=(a))
 #endif
 
 
@@ -473,12 +516,19 @@ int strcmpi(const char* str1, const char* str2);
 int strncmpi(const char* str1, const char* str2, size_t n);
 int u32cmp(const void *a, const void *b); 
 
+void biosigERROR(HDRTYPE *hdr, enum B4C_ERROR errnum, const char *errmsg);
+/*
+	sets the local and the (deprecated) global error variables B4C_ERRNUM and B4C_ERRMSG
+	the global error variables are kept for backwards compatibility.
+*/
+
+
 //double strtod (const char *nptr, char **endptr); // no locale dependency 
 
 /*
 	some important functions used internally, 
-	the interface for these functios is a bit clumsy and is 
-	(therefore) not 
+	the interface for these functios is a bit clumsy and are
+	therefore not exported to standard user applications. 
 */
 
 void struct2gdfbin(HDRTYPE *hdr);
@@ -508,14 +558,6 @@ int NumberOfChannels(HDRTYPE *hdr);
  ------------------------------------------------------------------------*/
 
 void FreeGlobalEventCodeTable();
-void FreeTextEvent(HDRTYPE* hdr,size_t N_EVENT, char* annotation);
-/*------------------------------------------------------------------------
-	adds free text annotation to event table    
-	the EVENT.TYP is identified from the table EVENT.CodeDesc
-	if annotations is not listed in CodeDesc, it is added to CodeDesc
-	The table is limited to 256 entries, because the table EventCodes
-	allows only codes 0-255 as user specific entry.
-  ------------------------------------------------------------------------*/
 
 size_t	sread_raw(size_t START, size_t LEN, HDRTYPE* hdr, char flag);
 /* sread_raw: 
