@@ -1,5 +1,5 @@
 /*
-% $Id: biosig.h,v 1.140 2009/04/09 15:06:55 schloegl Exp $
+% $Id$
 % Copyright (C) 2005,2006,2007,2008,2009,2010,2011 Alois Schloegl <alois.schloegl@gmail.com>
 % This file is part of the "BioSig for C/C++" repository
 % (biosig4c++) at http://biosig.sf.net/
@@ -34,6 +34,7 @@
 /**                 DEFINITIONS, TYPEDEFS AND MACROS                       **/
 /**                                                                        **/
 /****************************************************************************/
+
 #ifndef __BIOSIG_EXT_H__
 #define __BIOSIG_EXT_H__
 
@@ -51,15 +52,8 @@ typedef char			int8_t;
 #else
     #include <stdint.h>
 #endif
-
-
-#ifdef WITH_HDF5
-    #include <hdf5.h>
-#endif
-#ifdef WITH_NIFTI
-    #include <nifti1.h>
-#endif
-
+#include <stdio.h>
+#include <time.h>
 
 #ifdef __cplusplus
 #define EXTERN_C extern "C"
@@ -67,34 +61,22 @@ typedef char			int8_t;
 #define EXTERN_C
 #endif
 
-/*
-	Including ZLIB enables reading gzipped files (they are decompressed on-the-fly)
-	The output files can be zipped, too.
- */
-
-#ifdef WITH_ZLIB
-#include <zlib.h>
-#ifndef ZLIB_H
-    #if defined(__MINGW64__)
-	#include "win64/zlib/zlib.h"
-    #elif defined(__MINGW32__)
-	#include "win32/zlib/include/zlib.h"
-    #endif
+#ifdef __GNUC__
+#define ATT_ALI __attribute__ ((aligned (8)))	/* Matlab v7.3+ requires 8 byte alignment*/
+#define ATT_DEPREC __attribute__ ((deprecated))
+#else
+/* 
+	TODO (FIXME): 
+	not clear whether this alignment definition is equivalent to the one above:
+	This might be crucial for a mixed compiler environment 
+	e.g. libbiosig compiled with MinGW used in a MS VC++ project
+	Nevertheless, there are too many GNUC dependencies, so it probably will not 
+	compile with anything else but GNUC.
+*/
+#pragma pack(8)	
+#define ATT_ALI
+#define ATT_DEPREC
 #endif
-#endif
-
-#ifdef WITH_CHOLMOD
-    #ifdef __APPLE__
-        #include <ufsparse/cholmod.h>
-    #else
-        #include <suitesparse/cholmod.h>
-    #endif
-#endif
-#ifdef WITH_GSL
-    #include <gsl/gsl_matrix_double.h>
-#endif
-#include <stdio.h>
-#include <time.h>
 
 /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 	biosig_data_type    data type of  internal data format
@@ -146,12 +128,12 @@ enum FileFormat {
 	BrainVision, BrainVisionVAmp, BrainVisionMarker, BZ2,
 	CDF, CFS, CFWB, CNT, CTF, DICOM, DEMG,
 	EBS, EDF, EEG1100, EEProbe, EEProbe2, EEProbeAvr, EGI,
-	EGIS, ELF, EMBLA, ePrime, ET_MEG, ETG4000, EVENT, EXIF,
+	EGIS, ELF, EMBLA, EMSA, ePrime, ET_MEG, ETG4000, EVENT, EXIF,
 	FAMOS, FEF, FITS, FLAC, GDF, GDF1,
 	GIF, GTF, GZIP, HDF, HL7aECG, HEKA, 
 	ISHNE, ITX, JPEG, JSON, Lexicor,
 	Matlab, MFER, MIDI, MIT, MM, MSI, MSVCLIB, MS_LNK, 
-	native, NeuroLoggerHEX, NetCDF, NEURON, NEX1, NIFTI, 
+	native, NeuroLoggerHEX, NetCDF, NEURON, NEV, NEX1, NIFTI, 
 	OGG, OpenXDF,
 	PBMA, PBMN, PDF, PDP, Persyst, PGMA, PGMB,
 	PLEXON, PNG, PNM, POLY5, PPMA, PPMB, PS,
@@ -160,24 +142,39 @@ enum FileFormat {
 	SPSS, STATA, SVG, SXI, SYNERGY,
 	TIFF, TMS32, TMSiLOG, TRC, UNIPRO, VRML, VTK,
 	WAV, WG1, WinEEG, WMF, XML, XPM,
-	Z, ZIP, ZIP2, 
+	Z, ZIP, ZIP2,                        
+        TDMS
 };
 
 
-extern int B4C_ERRNUM;
-extern const char *B4C_ERRMSG;
+/*
+   error handling should use error variables local to each HDR
+   otherwise, sopen() etc. is not re-entrant. 
+
+   Therefore, use of variables B4C_ERRNUM and B4C_ERRMSG is deprecated; 	
+   Instead, use errorBIOSIG for setting error status, and 
+   serror2(hdr), hdr->AS.B4C_ERRNUM, hdr->AS.B4C_ERRMSG for error handling.   
+
+ */
+extern int B4C_ERRNUM ATT_DEPREC;
+extern const char *B4C_ERRMSG ATT_DEPREC;
 
 #define BIOSIG_VERSION_MAJOR 1
-#define BIOSIG_VERSION_MINOR 3
-#define BIOSIG_PATCHLEVEL 0
+#define BIOSIG_VERSION_MINOR 4
+#define BIOSIG_PATCHLEVEL 1
 // for backward compatibility 
 #define BIOSIG_VERSION_STEPPING BIOSIG_PATCHLEVEL	
 #define BIOSIG_VERSION (BIOSIG_VERSION_MAJOR * 10000 + BIOSIG_VERSION_MINOR * 100 + BIOSIG_PATCHLEVEL)
 
+/*
+This part has moved into biosig-dev.h in v1.4.1, because VERBOSE_LEVEL is just 
+used for debugging and should not be exposed to common applications 
+#ifdef NDEBUG
+#define VERBOSE_LEVEL 0		// turn off debugging information
+#else
 extern int VERBOSE_LEVEL; 	// used for debugging
-//#define VERBOSE_LEVEL 7		// turn on debugging information
-//#define VERBOSE_LEVEL 0		// turn off debugging information
-
+#endif
+*/ 
 
 /****************************************************************************/
 /**                                                                        **/
@@ -208,6 +205,7 @@ extern int VERBOSE_LEVEL; 	// used for debugging
 typedef int64_t 		gdf_time; /* gdf time is represented in 64 bits */
 
 #define t_time2gdf_time(t)	((gdf_time)floor(ldexp(((double)(t))/86400.0 + 719529.0, 32)))
+#define gdf_time2t_time(t)	((ldexp(((double)(t)),-32) - 719529.0)*86400.0)
 #define	ntp_time2gdf_time(t)	((gdf_time)ldexp(ldexp(((double)(t)),-32)/86400 + 719529.0 - 70,32))
 #define	gdf_time2ntp_time(t)	((int64_t)ldexp((ldexp(((double)(t)),-32) - 719529.0 + 70) * 86400,32))
 
@@ -241,30 +239,16 @@ typedef int64_t 		nrec_t;	/* type for number of records */
 /*
 	This structure defines the header for each channel (variable header)
  */
+// TODO: change fixed length strings to dynamically allocated strings
 #define MAX_LENGTH_LABEL 	40	// TMS: 40
 #define MAX_LENGTH_TRANSDUCER 	80
-#define MAX_LENGTH_PHYSDIM 	20
+#define MAX_LENGTH_PHYSDIM      20	// DEPRECATED - DO NOT USE 
 #define MAX_LENGTH_PID	 	80  	// length of Patient ID: MFER<65, GDF<67, EDF/BDF<81, etc.
 #define MAX_LENGTH_RID		80	// length of Recording ID: EDF,GDF,BDF<80, HL7 ?
 #define MAX_LENGTH_NAME 	132	// max length of personal name: MFER<=128, EBS<=33*4
 #define MAX_LENGTH_MANUF 	128	// max length of manufacturer field: MFER<128
 #define MAX_LENGTH_TECHNICIAN 	128	// max length of manufacturer field: SCP<41
 
-#ifdef __GNUC__
-#define ATT_ALI __attribute__ ((aligned (8)))	/* Matlab v7.3+ requires 8 byte alignment*/
-#define ATT_DEPREC __attribute__ ((deprecated))
-#else
-/* 
-	TODO (FIXME): 
-	not clear whether this alignment definition is equivalent to the one above:
-	This might be crucial for a mixed compiler environment 
-	e.g. libbiosig compiled with MinGW used in a MS VC++ project
-*/
-#pragma pack(8)	
-#define ATT_ALI
-#define ATT_DEPREC
-#endif
-
 typedef struct CHANNEL_STRUCT {
 	double 		PhysMin ATT_ALI;	/* physical minimum */
 	double 		PhysMax ATT_ALI;	/* physical maximum */
@@ -273,14 +257,12 @@ typedef struct CHANNEL_STRUCT {
 	double		Cal 	ATT_ALI;	/* gain factor */
 	double		Off 	ATT_ALI;	/* bias */
 
-	char		OnOff	ATT_ALI;
 	char		Label[MAX_LENGTH_LABEL+1] ATT_ALI; 	/* Label of channel */
+	char		OnOff	ATT_ALI;
 	uint16_t	LeadIdCode ATT_ALI;	/* Lead identification code */
 	char 		Transducer[MAX_LENGTH_TRANSDUCER+1] ATT_ALI;	/* transducer e.g. EEG: Ag-AgCl electrodes */
-	char 		PhysDim[MAX_LENGTH_PHYSDIM+1] ATT_ALI ATT_DEPREC;	/* physical dimension */
-			/*PhysDim is now obsolete - use function PhysDim(PhysDimCode,PhysDimText) instead */
-	uint16_t	PhysDimCode ATT_ALI;	/* code for physical dimension */
-	/* char* 	PreFilt;	// pre-filtering */
+        char            PhysDim[MAX_LENGTH_PHYSDIM+1] ATT_ALI ATT_DEPREC;       /* DONOT USE - use PhysDim3(PhysDimCode) instead */
+	uint16_t	PhysDimCode ATT_ALI;	/* code for physical dimension - PhysDim3(PhysDimCode) returns corresponding string */
 
 	float 		TOffset 	ATT_ALI;	/* time delay of sampling */
 	float 		LowPass		ATT_ALI;	/* lowpass filter */
@@ -294,36 +276,37 @@ typedef struct CHANNEL_STRUCT {
 	float 		fZ        	ATT_ALI;   	/* ICG probe frequency, defined only if PhysDim = _Ohm */
 	} ATT_ALI;
 
-	uint16_t 	GDFTYP 		ATT_ALI;	/* data type */
+	/* this part should not be used by application programs */
+	uint8_t*	bufptr		ATT_ALI;	/* pointer to buffer: NRec<=1 and bi,bi8 not used */
 	uint32_t 	SPR 		ATT_ALI;	/* samples per record (block) */
 	uint32_t	bi 		ATT_ALI;	/* start byte (byte index) of channel within data block */
 	uint32_t	bi8 		ATT_ALI;	/* start bit  (bit index) of channel within data block */
-	uint8_t*	bufptr		ATT_ALI;	/* pointer to buffer: NRec<=1 and bi,bi8 not used */
+	uint16_t 	GDFTYP 		ATT_ALI;	/* data type */
 } CHANNEL_TYPE	ATT_ALI;
 
 
 /*
 	This structure defines the general (fixed) header
 */
-typedef struct {
+typedef struct HDR_STRUCT {
 
-	enum FileFormat TYPE 	 ATT_ALI; 	/* type of file format */
-	float 		VERSION  ATT_ALI;	/* GDF version number */
 	char* 	        FileName ATT_ALI;       /* FileName - dynamically allocated, local copy of file name */
+	float 		VERSION  ATT_ALI;	/* GDF version number */
+	enum FileFormat TYPE 	 ATT_ALI; 	/* type of file format */
 
 	struct {
 		size_t 			size[2] ATT_ALI; /* size {rows, columns} of data block	 */
 		biosig_data_type* 	block ATT_ALI; 	 /* data block */
 	} data ATT_ALI;
 
-	uint32_t 	HeadLen ATT_ALI;	/* length of header in bytes */
-	uint16_t 	NS 	ATT_ALI;	/* number of channels */
-	uint32_t 	SPR 	ATT_ALI;	/* samples per block (when different sampling rates are used, this is the LCM(CHANNEL[..].SPR) */
-	nrec_t  	NRec 	ATT_ALI;	/* number of records/blocks -1 indicates length is unknown. */
-	double 		SampleRate ATT_ALI;	/* Sampling rate */
 	uint8_t 	IPaddr[16] ATT_ALI; 	/* IP address of recording device (if applicable) */
-	uint32_t  	LOC[4] 	ATT_ALI;	/* location of recording according to RFC1876 */
+	double 		SampleRate ATT_ALI;	/* Sampling rate */
+	nrec_t  	NRec 	ATT_ALI;	/* number of records/blocks -1 indicates length is unknown. */
 	gdf_time 	T0 	ATT_ALI; 	/* starttime of recording */
+	uint32_t 	HeadLen ATT_ALI;	/* length of header in bytes */
+	uint32_t 	SPR 	ATT_ALI;	/* samples per block (when different sampling rates are used, this is the LCM(CHANNEL[..].SPR) */
+	uint32_t  	LOC[4] 	ATT_ALI;	/* location of recording according to RFC1876 */
+	uint16_t 	NS 	ATT_ALI;	/* number of channels */
 	int16_t 	tzmin 	ATT_ALI;	/* time zone (minutes of difference to UTC */
 
 #ifdef CHOLMOD_H
@@ -335,15 +318,15 @@ typedef struct {
 
 	/* Patient specific information */
 	struct {
+		gdf_time 	Birthday; 	/* Birthday of Patient */
+		// 		Age;		// the age is HDR.T0 - HDR.Patient.Birthday, even if T0 and Birthday are not known
+		uint16_t	Headsize[3]; 	/* circumference, nasion-inion, left-right mastoid in millimeter;  */
 		char		Name[MAX_LENGTH_NAME+1]; /* because for privacy protection it is by default not supported, support is turned on with FLAG.ANONYMOUS */
 //		char*		Name;	// because for privacy protection it is by default not supported, support is turned on with FLAG.ANONYMOUS
 		char		Id[MAX_LENGTH_PID+1];	/* patient identification, identification code as used in hospital  */
 		uint8_t		Weight;		/* weight in kilograms [kg] 0:unkown, 255: overflow  */
 		uint8_t		Height;		/* height in centimeter [cm] 0:unkown, 255: overflow  */
 		//		BMI;		// the body-mass index = weight[kg]/height[m]^2
-		gdf_time 	Birthday; 	/* Birthday of Patient */
-		// 		Age;		// the age is HDR.T0 - HDR.Patient.Birthday, even if T0 and Birthday are not known
-		uint16_t	Headsize[3]; 	/* circumference, nasion-inion, left-right mastoid in millimeter;  */
 		/* Patient classification */
 		int8_t	 	Sex;		/* 0:Unknown, 1: Male, 2: Female */
 		int8_t		Handedness;	/* 0:Unknown, 1: Right, 2: Left, 3: Equal */
@@ -367,11 +350,11 @@ typedef struct {
 				SCP: section1, tag14,
 				MFER: tag23:  "Manufacturer^model^version number^serial number"
 			*/
-			char	_field[MAX_LENGTH_MANUF+1];	/* buffer */
 			const char*	Name;
 			const char*	Model;
 			const char*	Version;
 			const char*	SerialNumber;
+			char	_field[MAX_LENGTH_MANUF+1];	/* buffer */
 		} Manufacturer;
 	} ID ATT_ALI;
 
@@ -384,12 +367,12 @@ typedef struct {
 	/* EVENTTABLE */
 	struct {
 		double  	SampleRate ATT_ALI;	/* for converting POS and DUR into seconds  */
-		uint32_t  	N ATT_ALI;	/* number of events */
 		uint16_t 	*TYP ATT_ALI;	/* defined at http://cvs.sourceforge.net/viewcvs.py/biosig/biosig/t200/eventcodes.txt?view=markup */
 		uint32_t 	*POS ATT_ALI;	/* starting position [in samples] using a 0-based indexing */
 		uint32_t 	*DUR ATT_ALI;	/* duration [in samples] */
 		uint16_t 	*CHN ATT_ALI;	/* channel number; 0: all channels  */
-		char		**CodeDesc ATT_ALI;	/* describtion of "free text"/"user specific" events (encoded with TYP=0..255 */
+		const char*	*CodeDesc ATT_ALI;	/* describtion of "free text"/"user specific" events (encoded with TYP=0..255 */
+		uint32_t  	N ATT_ALI;	/* number of events */
 		uint16_t	LenCodeDesc ATT_ALI;	/* length of CodeDesc Table */
 	} EVENT ATT_ALI;
 
@@ -399,7 +382,6 @@ typedef struct {
 		char		ANONYMOUS; 	/* 1: anonymous mode, no personal names are processed */
 		char		ROW_BASED_CHANNELS;     /* 0: column-based data [default]; 1: row-based data */
 		char		TARGETSEGMENT; /* in multi-segment files (like Nihon-Khoden, EEG1100), it is used to select a segment */
-		char		CNT32;		/* 0: assume CNT format is 16 bit [default], <>0: CNT format is 32 bit*/
 	} FLAG ATT_ALI;
 
 	CHANNEL_TYPE 	*CHANNEL ATT_ALI;
@@ -428,6 +410,7 @@ typedef struct {
 
 	/*	internal variables (not public)  */
 	struct {
+		const char*	B4C_ERRMSG;	/* error message */
 //		char 		PID[MAX_LENGTH_PID+1];	// use HDR.Patient.Id instead
 //		char* 		RID;		// recording identification
 		uint32_t 	bpb;  		/* total bytes per block */
@@ -436,15 +419,17 @@ typedef struct {
 		uint8_t*	Header;
 		uint8_t*	rawEventData;
 		uint8_t*	rawdata; 	/* raw data block */
-		char		flag_collapsed_rawdata; /* 0 if rawdata contain obsolete channels, too. 	*/
 		size_t		first;		/* first block loaded in buffer - this is equivalent to hdr->FILE.POS */
 		size_t		length;		/* number of block(s) loaded in buffer */
 		uint8_t*	auxBUF;  	/* auxillary buffer - used for storing EVENT.CodeDesc, MIT FMT infor, alpha:rawdata header */
-		uint32_t	SegSel[5];	/* segment selection in a hirachical data formats, e.g. sweeps in HEKA/PatchMaster format */
 		char*		bci2000;
+		uint32_t	SegSel[5];	/* segment selection in a hirachical data formats, e.g. sweeps in HEKA/PatchMaster format */
+		enum B4C_ERROR	B4C_ERRNUM;	/* error code */
+		char		flag_collapsed_rawdata; /* 0 if rawdata contain obsolete channels, too. 	*/
 	} AS ATT_ALI;
 
 	void *aECG;				/* used as an pointer to (non-standard) auxilary information - mostly used for hacks */
+	uint64_t viewtime; 			/* used by edfbrowser */
 
 } HDRTYPE;
 
@@ -555,6 +540,13 @@ size_t	sread(biosig_data_type* DATA, size_t START, size_t LEN, HDRTYPE* hdr);
 	the number of columns and rows is available from
 	hdr->data.size[0] and hdr->data.size[1] respectively.
 
+	Channels k with (hdr->CHANNEL[k].SPR==0) are interpreted as sparsely
+	sampled channels [for details see specification ofGDF v2 or larger]. 
+	The sample values are also returned in DATA the corresponding
+  	sampling time, the values in between the sparse sampling times are
+	set to DigMin. (Applying the flags UCAL and OVERFLOWDETECTION will 
+	convert this into PhysMin and NaN, resp. see below). 
+
 	The following flags will influence the result.
  	hdr->FLAG.UCAL = 0 	scales the data to its physical values
  	hdr->FLAG.UCAL = 1 	does not apply the scaling factors
@@ -603,8 +595,20 @@ long int stell(HDRTYPE* hdr);
 /*	returns position of file point in segments
  --------------------------------------------------------------- */
 
+#ifndef  ONLYGDF
+int 	serror() ATT_DEPREC;
+/*	handles errors; it reports whether an error has occured.
+ *	if yes, an error message is displayed, and the error status is reset.
+ * 	the return value is 0 if no error has occured, otherwise the error code
+ *	is returned.
+ *  IMPORTANT NOTE:	
+ *	serror() uses the global error variables B4C_ERRNUM and B4C_ERRMSG, 
+ *	which is not re-entrant, because two opened files share the same 
+ *	error variables.  
+ --------------------------------------------------------------- */
+#endif //ONLYGDF
 
-int 	serror();
+int 	serror2(HDRTYPE* hdr);
 /*	handles errors; it reports whether an error has occured.
  *	if yes, an error message is displayed, and the error status is reset.
  * 	the return value is 0 if no error has occured, otherwise the error code
@@ -624,17 +628,6 @@ int 	cachingWholeFile(HDRTYPE* hdr);
  --------------------------------------------------------------- */
 
 
-int	hdr2ascii(HDRTYPE* hdr,FILE *fid, int VERBOSITY);
-/*	writes the header information is ascii format the stream defined by fid
- *	Typically fid is stdout. VERBOSITY defines how detailed the information is.
- *	VERBOSITY=0 or 1 report just some basic information,
- *	VERBOSITY=2 reports als the channel information
- *	VERBOSITY=3 provides in addition the event table.
- *	VERBOSITY=8 for debugging 
- *	VERBOSITY=9 for debugging
- *	VERBOSITY=-1 header and event table is shown in JSON format
- --------------------------------------------------------------- */
-
 int RerefCHANNEL(HDRTYPE *hdr, void *ReRef, char rrtype);
 /* rerefCHAN
         defines rereferencing of channels,
@@ -665,22 +658,9 @@ enum FileFormat GetFileTypeFromString(const char *);
  --------------------------------------------------------------- */
 
 
-uint16_t PhysDimCode(const char* PhysDim0);
-/* Encodes  Physical Dimension as 16bit integer according to
-   ISO/IEEE 11073-10101:2004 Vital Signs Units of Measurement
- --------------------------------------------------------------- */
-
-char* PhysDim(uint16_t PhysDimCode, char *PhysDimText);
-/* converts HDR.CHANNEL[k].PhysDimCode into a readable Physical Dimension
-   the memory for PhysDim must be preallocated, its maximum length is
-   defined by (MAX_LENGTH_PHYSDIM+1)
- --------------------------------------------------------------- */
-
-double PhysDimScale(uint16_t PhysDimCode);
-/* returns scaling factor of physical dimension
-	e.g. 0.001 for milli, 1000 for kilo etc.
- --------------------------------------------------------------- */
-
+/* =============================================================
+	utility functions for handling of event table 
+   ============================================================= */
 
 void sort_eventtable(HDRTYPE *hdr);
 /* sort event table with respect to hdr->EVENT.POS
@@ -695,6 +675,79 @@ void convert4to2_eventtable(HDRTYPE *hdr);
 	all CHN[k] must be 0
   -------------------------------------------------------------- */
 
+const char* GetEventDescription(HDRTYPE *hdr, size_t n);
+/* returns clear text description of n-th event, 
+   considers also user-defined events. 
+  -------------------------------------------------------------- */
+
+void FreeTextEvent(HDRTYPE* hdr, size_t N, const char* annotation);
+/*------------------------------------------------------------------------
+	adds free text annotation to event table for the N-th event.   
+	the EVENT.TYP[N] is identified from the table EVENT.CodeDesc
+	if annotations is not listed in CodeDesc, it is added to CodeDesc
+	The table is limited to 256 entries, because the table EventCodes
+	allows only codes 0-255 as user specific entry. If the description
+	table contains more than 255 entries, an error is set. 
+  ------------------------------------------------------------------------*/
+
+
+/* =============================================================
+	utility functions for handling of physical dimensons
+   ============================================================= */
+
+uint16_t PhysDimCode(const char* PhysDim);
+/* Encodes  Physical Dimension as 16bit integer according to
+   ISO/IEEE 11073-10101:2004 Vital Signs Units of Measurement
+ --------------------------------------------------------------- */
+
+char* PhysDim(uint16_t PhysDimCode, char *PhysDimText);
+/* DEPRECATED: USE INSTEAD PhysDim3(uint16_t PhysDimCode) 
+   It's included just for backwards compatibility
+   converts HDR.CHANNEL[k].PhysDimCode into a readable Physical Dimension
+   the memory for PhysDim must be preallocated, its maximum length is
+   defined by (MAX_LENGTH_PHYSDIM+1)
+ --------------------------------------------------------------- */
+
+const char* PhysDim3(uint16_t PhysDimCode);
+/* converts PhysDimCode into a readable Physical Dimension
+ --------------------------------------------------------------- */
+
+double PhysDimScale(uint16_t PhysDimCode);
+/* returns scaling factor of physical dimension
+	e.g. 0.001 for milli, 1000 for kilo etc.
+ --------------------------------------------------------------- */
+
+
+/* =============================================================
+	printing of header information 	
+   ============================================================= */
+
+int	hdr2ascii(HDRTYPE* hdr, FILE *fid, int VERBOSITY);
+/*	writes the header information is ascii format the stream defined by fid
+ *	Typically fid is stdout. VERBOSITY defines how detailed the information is.
+ *	VERBOSITY=0 or 1 report just some basic information,
+ *	VERBOSITY=2 reports als the channel information
+ *	VERBOSITY=3 provides in addition the event table.
+ *	VERBOSITY=8 for debugging 
+ *	VERBOSITY=9 for debugging
+ *	VERBOSITY=-1 header and event table is shown in JSON format
+ --------------------------------------------------------------- */
+
+int hdr2json (HDRTYPE *hdr, FILE *fid) ATT_DEPREC; 
+int fprintf_hdr2json(FILE *stream, HDRTYPE* hdr);
+/* prints header in json format into stream; 
+   hdr2json is the old form and deprecated, 
+   use fprintf_hdr2json instead
+ --------------------------------------------------------------- */
+
+int asprintf_hdr2json(char **str, HDRTYPE* hdr);
+/* prints header in json format into *str; 
+   memory for str is automatically allocated and must be freed 
+   after usage. 
+ --------------------------------------------------------------- */
+
+
+
 #ifdef __cplusplus
 }
 #endif
@@ -707,3 +760,4 @@ void convert4to2_eventtable(HDRTYPE *hdr);
 /****************************************************************************/
 
 #endif	/* __BIOSIG_EXT_H__ */
+
