@@ -178,7 +178,6 @@ void mexFunction(
 	double		*ChanList=NULL;
 	int		NS = -1;
 	char		FlagOverflowDetection = 1, FlagUCAL = 0;
-	char		FLAG_CNT32 = 0;
 	int		argSweepSel = -1;
 	
 #ifdef CHOLMOD_H
@@ -202,7 +201,6 @@ void mexFunction(
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan)\n\t\tchan must be sorted in ascending order\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,ReRef)\n\t\treref is a (sparse) matrix for rerefencing\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'...')\n");
-		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'CNT32')\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'OVERFLOWDETECTION:ON')\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'OVERFLOWDETECTION:OFF')\n");
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'UCAL:ON')\n");
@@ -212,7 +210,6 @@ void mexFunction(
 		mexPrintf("\t[s,HDR]=mexSLOAD(f,chan,'SWEEP',[NE, NG, NS])\n");
 		mexPrintf("   Input:\n\tf\tfilename\n");
 		mexPrintf("\tchan\tlist of selected channels; 0=all channels [default]\n");
-		mexPrintf("\tCNT32: needed to read 32bit CNT files \n");
 		mexPrintf("\tUCAL\tON: do not calibrate data; default=OFF\n");
 //		mexPrintf("\tOUTPUT\tSINGLE: single precision; default='double'\n");
 		mexPrintf("\tOVERFLOWDETECTION\tdefault = ON\n\t\tON: values outside dynamic range are not-a-number (NaN)\n");
@@ -267,7 +264,7 @@ void mexFunction(
 			if (k==0)			
 				FileName = mxArrayToString(prhs[k]);
 			else if (!strcmp(mxArrayToString(prhs[k]), "CNT32"))
-				FLAG_CNT32 = 1;
+				; // obsolete - supported for backwards compatibility
 			else if (!strcmp(mxArrayToString(prhs[k]), "OVERFLOWDETECTION:ON"))
 				FlagOverflowDetection = 1;
 			else if (!strcmp(mxArrayToString(prhs[k]), "OVERFLOWDETECTION:OFF"))
@@ -300,7 +297,6 @@ void mexFunction(
 	hdr = constructHDR(0,0);
 	hdr->FLAG.OVERFLOWDETECTION = FlagOverflowDetection; 
 	hdr->FLAG.UCAL = FlagUCAL;
-	hdr->FLAG.CNT32 = FLAG_CNT32;
 #ifdef CHOLMOD_H
 	hdr->FLAG.ROW_BASED_CHANNELS = (rr!=NULL); 
 #else 	
@@ -324,8 +320,8 @@ void mexFunction(
 	hdr = sopen(FileName, "r", hdr);
 /*
 #ifdef WITH_PDP 
-	if (B4C_ERRNUM) {
-		B4C_ERRNUM = 0;
+	if (hdr->AS.B4C_ERRNUM) {
+		hdr->AS.B4C_ERRNUM = 0;
 		sopen_pdp_read(hdr);
 	}	
 #endif
@@ -333,7 +329,7 @@ void mexFunction(
 	if (VERBOSE_LEVEL>7) 
 		mexPrintf("121: sopen done\n");
 
-	if ((status=serror())) {
+	if ((status=serror2(hdr))) {
 
 		const char* fields[]={"TYPE","VERSION","FileName","FLAG","ErrNum","ErrMsg"};
 		HDR = mxCreateStructMatrix(1, 1, 6, fields);
@@ -364,9 +360,9 @@ void mexFunction(
 		    else if (status==B4C_FORMAT_UNKNOWN)
 			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not known.\n",FileName,GetFileTypeString(hdr->TYPE));	/* Flawfinder: ignore *** sufficient memory is allocated above */
 		    else if (status==B4C_FORMAT_UNSUPPORTED)
-			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", FileName, GetFileTypeString(hdr->TYPE), B4C_ERRMSG);	/* Flawfinder: ignore *** sufficient memory is allocated above */
+			sprintf(msg,"Error mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", FileName, GetFileTypeString(hdr->TYPE), hdr->AS.B4C_ERRMSG);	/* Flawfinder: ignore *** sufficient memory is allocated above */
 		    else 	
-			sprintf(msg,"Error %i mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", status, FileName, GetFileTypeString(hdr->TYPE), B4C_ERRMSG); 	/* Flawfinder: ignore *** sufficient memory is allocated above */
+			sprintf(msg,"Error %i mexSLOAD: Cannot open file %s - format %s not supported [%s].\n", status, FileName, GetFileTypeString(hdr->TYPE), hdr->AS.B4C_ERRMSG); 	/* Flawfinder: ignore *** sufficient memory is allocated above */
 			
 		    mxSetField(HDR,0,"ErrMsg",mxCreateString(msg));
 		    free(msg);
@@ -452,7 +448,7 @@ void mexFunction(
                 hdr->Calib = NULL; 
         }                
 #endif 
-	if ((status=serror())) return;  
+	if ((status=serror2(hdr))) return;  
 
 	if (VERBOSE_LEVEL>7) 
 		fprintf(stderr,"\n[129] SREAD/SCLOSE on %s successful [%i,%i] [%Li,%i] %i.\n",hdr->FileName,hdr->data.size[0],hdr->data.size[1],hdr->NRec,count,NS);
@@ -554,9 +550,7 @@ void mexFunction(
 			mxSetCell(Label,k1,mxCreateString(hdr->CHANNEL[k].Label));
 			mxSetCell(Transducer,k1,mxCreateString(hdr->CHANNEL[k].Transducer));
 			
-			char p[MAX_LENGTH_PHYSDIM+1];
-			PhysDim(hdr->CHANNEL[k].PhysDimCode,p);			
-			mxSetCell(PhysDim1,k1,mxCreateString(p));
+			mxSetCell(PhysDim1,k1,mxCreateString(PhysDim3(hdr->CHANNEL[k].PhysDimCode)));
 			k1++;
 		} 
 
